#include "DolibarrClient.h"
#include <WiFi.h>
#include <ArduinoJson.h>
#include <iostream>
#include "M5LCD.h"

HTTPClient *client = new HTTPClient();

DolibarrClient::DolibarrClient(struct DolibarrConfig dolibarr_config) : dolibarr(dolibarr_config) {
    #if defined(DEBUG)
        Serial.println(" --- Dolibarr configuration --- ");
        Serial.println((std::string("Base URL: ") + std::string(dolibarr_config.url)).c_str());
        Serial.println((std::string("Token: ") + std::string(dolibarr_config.api_key)).c_str());
    #endif
    this->initialize_http_client();
}

HTTPClient *DolibarrClient::build_url(const String& url) const {
    String clientUrl = this->dolibarr.url + url;
    client->begin(clientUrl);
    client->addHeader("Content-Type", "application/json");
    client->addHeader("DOLAPIKEY", this->dolibarr.api_key);
    #if defined(DEBUG)
        Serial.println("URL Request: " + clientUrl);
    #endif
    return client;
}

String replace_id(const char *str, const char *id) {
    String url(str);
    url.replace("{id}", id);
    return url;
}

models::Product DolibarrClient::get_product_by_factory_id(const char* factory_id) const {
    HTTPClient *client = this->build_url(replace_id("/products?sortorder=ASC&limit=1&sqlfilters=(t.accountancy_code_sell:like:'{id}')", factory_id).c_str());
    if (client->GET() == HTTP_CODE_OK) {
        DynamicJsonDocument doc(6144);
        DeserializationError error = deserializeJson(doc, client->getString().c_str());
        if (error) {
            Serial.println("ERROR: ");
            Serial.println(error.c_str());
            return {};
        }
        for (auto obj : doc.as<JsonArray>()) {
            models::Product product;
            product.date_creation = obj["date_creation"].as<std::string>();
            product.id = obj["id"].as<std::string>();
            product.entity = obj["entity"].as<std::string>();
            product.stock_reel = obj["stock_reel"].as<std::string>();
            product.label = obj["label"].as<std::string>();
            product.accountancy_code_sell = obj["accountancy_code_sell"].as<std::string>();
            product.accountancy_code_sell_export = obj["accountancy_code_sell_export"].as<std::string>();
            product.fk_default_warehouse = obj["fk_default_warehouse"].as<std::string>();
            return product;
        }
        return {};
    }
    return {};
}

models::Product *DolibarrClient::get_product_by_id(const char* id_product) const {
    HTTPClient *client = this->build_url(replace_id(API_GET_PRODUCT_URL, id_product).c_str());
    if (client->GET() == HTTP_CODE_OK) {
        DynamicJsonDocument doc(6144);
        DeserializationError error = deserializeJson(doc, client->getString().c_str());
        if (error) {
            return nullptr;
        }
        auto *product = new models::Product();
        product->date_creation = doc["date_creation"].as<std::string>();
        product->id = doc["id"].as<std::string>();
        product->entity = doc["entity"].as<std::string>();
        product->stock_reel = doc["stock_reel"].as<std::string>();
        product->label = doc["label"].as<std::string>();
        product->accountancy_code_sell = doc["accountancy_code_sell"].as<std::string>();
        product->accountancy_code_sell_export = doc["accountancy_code_sell_export"].as<std::string>();
        return product;
    }
    return nullptr;
}

std::vector<models::Warehouse> *DolibarrClient::list_warehouse() const {
    HTTPClient *client = this->build_url(API_LIST_WAREHOUSE_URL);
    if (client->GET() == HTTP_CODE_OK) {
        DynamicJsonDocument doc(8192);
        DeserializationError error = deserializeJson(doc, client->getString().c_str());
        if (error) {
            Serial.println("ERROR: ");
            Serial.println(error.c_str());
            return nullptr;
        }
        auto *warehouses = new std::vector<models::Warehouse>();
        for (auto obj : doc.as<JsonArray>()) {
            models::Warehouse warehouse = {};
            warehouse.id = obj["id"].as<std::string>();
            warehouse.label = obj["label"].as<std::string>();
            warehouses->push_back(warehouse);
        }
        return warehouses;
    }
    return nullptr;
}

int DolibarrClient::create_movement(models::CreateProductStock stock) const {
    HTTPClient *client = this->build_url(API_CREATE_STOCKS_MOVEMENTS_URL);
    DynamicJsonDocument doc(4096);
    std::string result;
    doc["product_id"] = stock.product_id;
    doc["warehouse_id"]  = stock.warehouse_id;
    doc["qty"] = stock.qty;
    doc["movementlabel"] = "T-IOT - Warehouse GUI";
    doc["movementcode"] = "M" + stock.product_id + "-W" + stock.warehouse_id;
    serializeJson(doc, result);
    Serial.println(result.c_str());
    if (client->POST(result.c_str()) == HTTP_CODE_OK) {
        return 0;
    }
    return -1;
}

int DolibarrClient::initialize_http_client() {
    this->httpClient = new HTTPClient();
    return 0;
}